#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
import json
import os
import random
random.seed(int(os.getenv("SEED"), 16))
from prjxray import util
from prjxray import verilog
from prjxray.db import Database
from prjxray.lut_maker import LutMaker

WRITE_MODES = ("WRITE_FIRST", "NO_CHANGE", "READ_FIRST")


def gen_sites():
    db = Database(util.get_db_root(), util.get_part())
    grid = db.grid()
    for tile_name in sorted(grid.tiles()):
        loc = grid.loc_of_tilename(tile_name)
        gridinfo = grid.gridinfo_at_loc(loc)

        if gridinfo.tile_type not in ['BRAM_L', 'BRAM_R']:
            continue

        sites = {}
        for site_name, site_type in gridinfo.sites.items():
            sites[site_type] = site_name

        yield tile_name, sites


def ramb18(tile_name, luts, lines, sites):
    """ RAMB18E1 in either top or bottom site. """

    params = {}
    params['tile'] = tile_name
    params['Y0_IN_USE'] = random.randint(0, 1) == 1
    params['Y1_IN_USE'] = not params['Y0_IN_USE']
    params['FIFO_Y0_IN_USE'] = False
    params['FIFO_Y1_IN_USE'] = False

    if params['Y0_IN_USE']:
        site = sites['FIFO18E1']
    elif params['Y1_IN_USE']:
        site = sites['RAMB18E1']
    else:
        assert False

    lines.append(
        '''
        (* KEEP, DONT_TOUCH, LOC = "{site}" *)
        RAMB18E1 #(
            ) bram_{site} (
            );
        '''.format(site=site))

    return params


def ramb18_2x(tile_name, luts, lines, sites):
    """ RAMB18E1 in both top and bottom site. """

    params = {}
    params['tile'] = tile_name
    params['Y0_IN_USE'] = True
    params['Y1_IN_USE'] = True
    params['FIFO_Y0_IN_USE'] = False
    params['FIFO_Y1_IN_USE'] = False

    lines.append(
        '''
        (* KEEP, DONT_TOUCH, LOC = "{top_site}" *)
        RAMB18E1 #(
            ) bram_{top_site} (
            );
        (* KEEP, DONT_TOUCH, LOC = "{bottom_site}" *)
        RAMB18E1 #(
            ) bram_{bottom_site} (
            );
        '''.format(
            top_site=sites['FIFO18E1'],
            bottom_site=sites['RAMB18E1'],
        ))

    return params


def ramb36(tile_name, luts, lines, sites):
    """ RAMB36 consuming entire tile. """

    params = {}
    params['tile'] = tile_name
    params['Y0_IN_USE'] = True
    params['Y1_IN_USE'] = True
    params['FIFO_Y0_IN_USE'] = False
    params['FIFO_Y1_IN_USE'] = False

    site = sites['RAMBFIFO36E1']

    lines.append(
        """
        wire [7:0] webwe_{site};
        wire [3:0] wea_{site};
        wire regce_{site};
        wire [15:0] rdaddr_{site};
        wire [15:0] wraddr_{site};
        """.format(site=site))

    for bit in range(15):
        lines.append(
            'assign rdaddr_{site}[{bit}] = {net};'.format(
                bit=bit, site=site, net=luts.get_next_output_net()))
        lines.append(
            'assign wraddr_{site}[{bit}] = {net};'.format(
                bit=bit, site=site, net=luts.get_next_output_net()))

    for bit in range(8):
        lines.append(
            'assign webwe_{site}[{bit}] = {net};'.format(
                bit=bit, site=site, net=luts.get_next_output_net()))

    for bit in range(4):
        lines.append(
            'assign wea_{site}[{bit}] = {net};'.format(
                bit=bit, site=site, net=luts.get_next_output_net()))

    lines.append(
        'assign regce_{site} = {net};'.format(
            bit=bit, site=site, net=luts.get_next_output_net()))

    do_reg = verilog.vrandbit()
    ram_mode = random.choice(('SDP', 'TDP'))
    READ_WIDTH_A = 0
    READ_WIDTH_B = 0

    if ram_mode == 'TDP':
        write_mode_a = random.choice(WRITE_MODES)
        write_mode_b = random.choice(WRITE_MODES)
        WRITE_WIDTH_A = 36
        WRITE_WIDTH_B = 36
    else:
        write_mode_a = 'WRITE_FIRST'
        write_mode_b = 'WRITE_FIRST'
        WRITE_WIDTH_A = 72
        WRITE_WIDTH_B = 72

    lines.append(
        '''
        (* KEEP, DONT_TOUCH, LOC = "{site}" *)
        RAMB36E1 #(
            .DOA_REG({doa_reg}),
            .DOB_REG({dob_reg}),
            .WRITE_MODE_A({write_mode_a}),
            .WRITE_MODE_B({write_mode_b}),
            .READ_WIDTH_A({READ_WIDTH_A}),
            .READ_WIDTH_B({READ_WIDTH_B}),
            .WRITE_WIDTH_A({WRITE_WIDTH_A}),
            .WRITE_WIDTH_B({WRITE_WIDTH_B}),
            .INIT_A(36'hFF_FFFF_FFFF),
            .SRVAL_A(36'hFF_FFFF_FFFF),
            .INIT_B(36'hFF_FFFF_FFFF),
            .SRVAL_B(36'hFF_FFFF_FFFF),
            .RAM_MODE({ram_mode})
            ) bram_{site} (
                .ADDRARDADDR(rdaddr_{site}),
                .ADDRBWRADDR(wraddr_{site}),
                .REGCEAREGCE(regce_{site}),
                .REGCEB(regce_{site}),
                .WEBWE(webwe_{site}),
                .WEA(wea_{site})
            );
        '''.format(
            site=site,
            doa_reg=do_reg,
            dob_reg=do_reg,
            write_mode_a=verilog.quote(write_mode_a),
            write_mode_b=verilog.quote(write_mode_b),
            ram_mode=verilog.quote(ram_mode),
            READ_WIDTH_A=READ_WIDTH_A,
            READ_WIDTH_B=READ_WIDTH_B,
            WRITE_WIDTH_A=WRITE_WIDTH_A,
            WRITE_WIDTH_B=WRITE_WIDTH_B,
        ))

    return params


def fifo18(tile_name, luts, lines, sites):
    """ FIFO18E1 without bottom RAMB site. """

    params = {}
    params['tile'] = tile_name
    params['Y0_IN_USE'] = True
    params['Y1_IN_USE'] = False
    params['FIFO_Y0_IN_USE'] = True
    params['FIFO_Y1_IN_USE'] = False

    lines.append(
        '''
        wire fifo_rst_{site};
        (* KEEP, DONT_TOUCH *)
        LUT6 fifo_lut_{site} (
            .O(fifo_rst_{site})
            );

        (* KEEP, DONT_TOUCH, LOC = "{site}" *)
        FIFO18E1 #(
            .ALMOST_EMPTY_OFFSET(8),
            .ALMOST_FULL_OFFSET(8),
            .DATA_WIDTH({data_width})
            ) fifo_{site} (
                .RST(fifo_rst_{site})
            );
        '''.format(
            site=sites['FIFO18E1'],
            data_width=random.choice((4, 9)),
        ))

    return params


def fifo18_ramb18(tile_name, luts, lines, sites):
    """ FIFO18E1 in top site and RAMB18E1 in bottom site. """

    params = {}
    params['tile'] = tile_name
    params['Y0_IN_USE'] = True
    params['Y1_IN_USE'] = True
    params['FIFO_Y0_IN_USE'] = True
    params['FIFO_Y1_IN_USE'] = False

    lines.append(
        '''
        wire fifo_rst_{fifo_site};
        (* KEEP, DONT_TOUCH *)
        LUT6 fifo_lut_{fifo_site} (
            .O(fifo_rst_{fifo_site})
            );

        (* KEEP, DONT_TOUCH, LOC = "{fifo_site}" *)
        FIFO18E1 #(
            .ALMOST_EMPTY_OFFSET(5),
            .ALMOST_FULL_OFFSET(5)
            ) fifo_{fifo_site} (
                .RST(fifo_rst_{fifo_site})
            );

        (* KEEP, DONT_TOUCH, LOC = "{ramb_site}" *)
        RAMB18E1 #(
            ) bram_{ramb_site} (
            );
        '''.format(
            fifo_site=sites['FIFO18E1'],
            ramb_site=sites['RAMB18E1'],
        ))

    return params


def fifo36(tile_name, luts, lines, sites):
    """ FIFO36E1 consuming entire tile. """

    params = {}
    params['tile'] = tile_name
    params['Y0_IN_USE'] = True
    params['Y1_IN_USE'] = True
    params['FIFO_Y0_IN_USE'] = True
    params['FIFO_Y1_IN_USE'] = True

    data_width = random.choice((4, 9))
    if data_width == 4:
        ALMOST_EMPTY_OFFSET = 8184
        ALMOST_FULL_OFFSET = 8184
    else:
        ALMOST_EMPTY_OFFSET = 4087
        ALMOST_FULL_OFFSET = 4087

    lines.append(
        '''
        wire fifo_rst_{site};
        (* KEEP, DONT_TOUCH *)
        LUT6 fifo_lut_{site} (
            .O(fifo_rst_{site})
            );

        (* KEEP, DONT_TOUCH, LOC = "{site}" *)
        FIFO36E1 #(
            .ALMOST_EMPTY_OFFSET({ALMOST_EMPTY_OFFSET}),
            .ALMOST_FULL_OFFSET({ALMOST_FULL_OFFSET}),
            .DATA_WIDTH({data_width}),
            .INIT(36'hFF_FFFF_FFFF),
            .SRVAL(36'hFF_FFFF_FFFF)
            ) fifo_{site} (
                .RST(fifo_rst_{site})
            );
        '''.format(
            site=sites['RAMBFIFO36E1'],
            data_width=data_width,
            ALMOST_EMPTY_OFFSET=ALMOST_EMPTY_OFFSET,
            ALMOST_FULL_OFFSET=ALMOST_FULL_OFFSET))

    return params


def main():
    print('''
module top();
    ''')

    luts = LutMaker()
    lines = []

    params_list = []
    for tile_name, sites in gen_sites():
        gen_fun = random.choice(
            (ramb18, ramb18_2x, ramb36, fifo18, fifo18_ramb18, fifo36))
        params_list.append(gen_fun(tile_name, luts, lines, sites))

    for lut in luts.create_wires_and_luts():
        print(lut)

    for l in lines:
        print(l)

    print("endmodule")

    with open('params.json', 'w') as f:
        json.dump(params_list, f, indent=2)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2021  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
import argparse
import yaml
import subprocess
import os
import re
import tempfile
import json
from prjxray.util import OpenSafeFile, db_root_arg, get_parts, set_part_resources


def main():
    """Tool to update the used resources by the fuzzers for each available part.

    Example:
        prjxray$ ./utils/update_resources.py artix7 --db-root database/artix7/
    """
    parser = argparse.ArgumentParser(
        description="Saves all resource information for a family.")

    parser.add_argument(
        'family',
        help="Name of the device family.",
        choices=['artix7', 'kintex7', 'zynq7', 'spartan7'])
    db_root_arg(parser)

    args = parser.parse_args()
    env = os.environ.copy()
    cwd = os.path.dirname(os.path.abspath(__file__))
    resource_path = os.path.join(
        os.getenv('XRAY_DIR'), 'settings', args.family)
    information = {}

    parts = get_parts(args.db_root)
    processed_parts = dict()
    for part in parts.keys():
        # Skip parts which differ only in the speedgrade, as they have the same pins
        fields = part.split("-")
        common_part = fields[0]
        if common_part in processed_parts:
            information[part] = processed_parts[common_part]
            continue

        print("Find pins for {}".format(part))
        env['XRAY_PART'] = part
        _, tmp_file = tempfile.mkstemp()
        # Asks with get_package_pins and different filters for pins with
        # specific properties.
        command = "env TMP_FILE={} {} -mode batch -source update_resources.tcl".format(
            tmp_file, env['XRAY_VIVADO'])
        result = subprocess.run(
            command.split(' '),
            check=True,
            env=env,
            cwd=cwd,
            stdout=subprocess.PIPE)

        with OpenSafeFile(tmp_file, "r") as fp:
            pins_json = json.load(fp)

        os.remove(tmp_file)

        clk_pins = pins_json["clk_pins"].split()
        data_pins = pins_json["data_pins"].split()
        pins = {
            0: clk_pins[0],
            1: data_pins[0],
            2: data_pins[int(len(data_pins) / 2)],
            3: data_pins[-1]
        }
        information[part] = {'pins': pins}
        processed_parts[common_part] = {'pins': pins}

    # Overwrites the <family>/resources.yaml file completly with new data
    set_part_resources(resource_path, information)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
import os
import random
random.seed(int(os.getenv("SEED"), 16))
from prjxray import util
from prjxray.db import Database

INT_TILE_TYPES = ['INT_L', 'INT_R']
HCLK_TILE_TYPES = ['HCLK_L', 'HCLK_R', 'HCLK_L_BOT_UTURN', 'HCLK_R_BOT_UTURN']


def get_int_column_roots(grid):
    for tile_name in sorted(grid.tiles()):
        loc = grid.loc_of_tilename(tile_name)
        gridinfo = grid.gridinfo_at_loc(loc)

        if gridinfo.tile_type not in INT_TILE_TYPES:
            continue

        next_gridinfo = grid.gridinfo_at_loc((loc.grid_x, loc.grid_y + 1))

        if next_gridinfo.tile_type in INT_TILE_TYPES:
            continue

        if next_gridinfo.tile_type in HCLK_TILE_TYPES:
            continue

        assert next_gridinfo.tile_type in [
            'B_TERM_INT', 'BRKH_INT', 'BRKH_B_TERM_INT'
        ], next_gridinfo.tile_type

        yield tile_name


def build_int_columns(grid):

    int_columns = {}

    for root_tile_name in get_int_column_roots(grid):
        assert root_tile_name not in int_columns
        int_columns[root_tile_name] = []

        tile_name = root_tile_name
        gridinfo = grid.gridinfo_at_tilename(tile_name)

        # Walk up INT column.
        while gridinfo.tile_type in INT_TILE_TYPES:
            int_columns[root_tile_name].append(tile_name)

            loc = grid.loc_of_tilename(tile_name)
            tile_name = grid.tilename_at_loc((loc.grid_x, loc.grid_y - 1))
            gridinfo = grid.gridinfo_at_tilename(tile_name)

            if gridinfo.tile_type in HCLK_TILE_TYPES:
                loc = grid.loc_of_tilename(tile_name)
                tile_name = grid.tilename_at_loc((loc.grid_x, loc.grid_y - 1))
                gridinfo = grid.gridinfo_at_tilename(tile_name)

        assert gridinfo.tile_type in [
            'T_TERM_INT', 'BRKH_INT', 'BRKH_TERM_INT'
        ], gridinfo.tile_type

    return int_columns


def pair_int_tiles(grid, int_tiles):
    tiles_left = set(int_tiles)

    while tiles_left:
        tile_name = tiles_left.pop()

        gridinfo = grid.gridinfo_at_tilename(tile_name)
        loc = grid.loc_of_tilename(tile_name)

        assert gridinfo.tile_type in INT_TILE_TYPES

        if gridinfo.tile_type == 'INT_L':
            other_int_tile = 'INT_R'
            other_loc = (loc.grid_x + 1, loc.grid_y)
        else:
            other_int_tile = 'INT_L'
            other_loc = (loc.grid_x - 1, loc.grid_y)

        paired_tile_name = grid.tilename_at_loc(other_loc)
        paired_gridinfo = grid.gridinfo_at_tilename(paired_tile_name)
        assert paired_gridinfo.tile_type == other_int_tile, paired_gridinfo.tile_type

        tiles_left.remove(paired_tile_name)

        yield sorted([tile_name, paired_tile_name])


def is_orphan_int_row(grid, int_l, int_r):
    """ Returns true if given INT pair have no adjcent sites. """

    loc = grid.loc_of_tilename(int_l)

    if grid.gridinfo_at_loc(
        (loc.grid_x - 1, loc.grid_y)).tile_type != 'INT_INTERFACE_L':
        return False

    if grid.gridinfo_at_loc(
        (loc.grid_x - 2, loc.grid_y)).tile_type != 'VFRAME':
        return False

    loc = grid.loc_of_tilename(int_r)

    if grid.gridinfo_at_loc(
        (loc.grid_x + 1, loc.grid_y)).tile_type != 'INT_INTERFACE_R':
        return False

    if grid.gridinfo_at_loc(
        (loc.grid_x + 2,
         loc.grid_y)).tile_type not in ['CLK_FEED', 'CLK_BUFG_REBUF', 'NULL']:
        return False

    return True


def gen_orphan_ints(grid):
    int_columns = build_int_columns(grid)

    for int_l_column, int_r_column in sorted(pair_int_tiles(
            grid, int_columns.keys())):
        found_site = False
        for int_l, int_r in zip(int_columns[int_l_column],
                                int_columns[int_r_column]):
            if not is_orphan_int_row(grid, int_l, int_r):
                found_site = True
                break

        if not found_site:
            yield int_columns[int_l_column], int_columns[int_r_column]


def write_params(params):
    pinstr = 'tile,val\n'
    for tile, (val) in sorted(params.items()):
        pinstr += '%s,%s\n' % (tile, val)
    open('params.csv', 'w').write(pinstr)


def build_cross_int(params, grid, int_l, int_r):
    """ Toggles INT_R.ER1BEG1.EE4END0 """

    loc = grid.loc_of_tilename(int_r)

    origin_tile = grid.tilename_at_loc((loc.grid_x - 4, loc.grid_y))
    origin_gridinfo = grid.gridinfo_at_tilename(origin_tile)
    assert origin_gridinfo.tile_type == 'CLBLL_R'
    origin_site = sorted(origin_gridinfo.sites.keys())[random.randint(0, 1)]

    dest_tile = grid.tilename_at_loc((loc.grid_x + 4, loc.grid_y))
    dest_gridinfo = grid.gridinfo_at_tilename(dest_tile)
    assert dest_gridinfo.tile_type == 'CLBLL_L'
    dest_site = sorted(dest_gridinfo.sites.keys())[0]

    dest2_tile = grid.tilename_at_loc((loc.grid_x + 4, loc.grid_y + 1))
    dest2_gridinfo = grid.gridinfo_at_tilename(dest2_tile)
    assert dest2_gridinfo.tile_type == 'CLBLL_L', dest2_gridinfo.tile_type
    dest2_site = sorted(dest2_gridinfo.sites.keys())[1]

    if random.randint(0, 1):
        dest_wire = 'origin_wire_{origin_site}'.format(origin_site=origin_site)
    else:
        dest_wire = '1'

    if random.randint(0, 1):
        dest_wire2 = 'origin_wire_{origin_site}'.format(
            origin_site=origin_site)
    else:
        dest_wire2 = '1'

    if random.randint(0, 1):
        dest_wire3 = 'origin_wire_{origin_site}'.format(
            origin_site=origin_site)
    else:
        dest_wire3 = '1'

    if random.randint(0, 1):
        dest_wire4 = 'origin_wire_{origin_site}'.format(
            origin_site=origin_site)
    else:
        dest_wire4 = '1'

    # origin_site.AQ -> dest_tile.D6 enables INT_R.ER1BEG1.EE4END0
    print(
        """
    // Force origin FF into A position with MUXF7_L and MUXF8
    wire origin_wire_{origin_site};

    wire f7_to_lo_{origin_site};
    wire lut_to_f7_{origin_site};
    (* KEEP, DONT_TOUCH, LOC = "{origin_site}" *)
    LUT6_L #(
    .INIT(0)
    ) lut_rom_{origin_site} (
            .I0(1),
            .I1(origin_wire_{origin_site}),
            .I2(0),
            .I3(1),
            .I4(1),
            .I5(1),
            .LO(lut_to_f7_{origin_site})
            );
    (* KEEP, DONT_TOUCH, LOC = "{origin_site}" *)
    MUXF7_D f7_{origin_site} (
        .I0(lut_to_f7_{origin_site}),
        .LO(f7_to_lo_{origin_site}),
        .O(origin_wire_{origin_site})
        );
    (* KEEP, DONT_TOUCH, LOC = "{origin_site}" *)
    MUXF8 f8_{origin_site} (
        .I1(f7_to_lo_{origin_site})
        );

    wire dest_wire_{dest_site};
    wire dest_wire2_{dest_site};

    wire d_lut_to_f7_{dest_site}, f7_to_f8_{dest_site};
    // Force destination LUT into D position with MUXF7_L and MUXF8
    (* KEEP, DONT_TOUCH, LOC = "{dest_site}" *)
    LUT6_L #(
    .INIT(0)
    ) d_lut_rom_{dest_site} (
            .I0(1),
            .I1(1),
            .I2(0),
            .I3(1),
            .I4(1),
            .I5(dest_wire_{dest_site}),
            .LO(d_lut_to_f7_{dest_site})
            );

    wire c_lut_to_f7_{dest_site};
    // Force destination LUT into C position with MUXF7_L and MUXF8
    (* KEEP, DONT_TOUCH, LOC = "{dest_site}" *)
    LUT6_L #(
    .INIT(0)
    ) c_lut_rom_{dest_site} (
            .I0(1),
            .I1(1),
            .I2(0),
            .I3(1),
            .I4(1),
            .I5(dest_wire2_{dest_site}),
            .LO(c_lut_to_f7_{dest_site})
            );
    (* KEEP, DONT_TOUCH, LOC = "{dest_site}" *)
    MUXF7_L f7_{dest_site} (
        .I0(d_lut_to_f7_{dest_site}),
        .I1(c_lut_to_f7_{dest_site}),
        .LO(f7_to_f8_{dest_site})
        );
    (* KEEP, DONT_TOUCH, LOC = "{dest_site}" *)
    MUXF8 f8_{dest_site} (
        .I0(f7_to_f8_{dest_site})
        );

    assign dest_wire_{dest_site} = {dest_wire};
    assign dest_wire2_{dest_site} = {dest_wire2};

    wire dest_wire3_{dest_site2};
    wire dest_wire4_{dest_site2};
    wire lut_to_f7_{dest_site2}, f7_to_f8_{dest_site2};
    // Force destination LUT into D position with MUXF7_L and MUXF8
    (* KEEP, DONT_TOUCH, LOC = "{dest_site2}" *)
    LUT6_L #(
    .INIT(0)
    ) d_lut_rom_{dest_site2} (
            .I0(dest_wire3_{dest_site2}),
            .I1(1),
            .I2(0),
            .I3(1),
            .I4(1),
            .I5(),
            .LO(lut_to_f7_{dest_site2})
            );

    // Force destination LUT into C position with MUXF7_L and MUXF8
    wire c_lut_to_f7_{dest_site2};
    (* KEEP, DONT_TOUCH, LOC = "{dest_site2}" *)
    LUT6_L #(
    .INIT(0)
    ) c_lut_rom_{dest_site2} (
            .I0(dest_wire4_{dest_site2}),
            .I1(1),
            .I2(0),
            .I3(1),
            .I4(1),
            .I5(1),
            .LO(c_lut_to_f7_{dest_site2})
            );
    (* KEEP, DONT_TOUCH, LOC = "{dest_site2}" *)
    MUXF7_L f7_{dest_site2} (
        .I0(lut_to_f7_{dest_site2}),
        .I1(c_lut_to_f7_{dest_site2}),
        .LO(f7_to_f8_{dest_site2})
        );
    (* KEEP, DONT_TOUCH, LOC = "{dest_site2}" *)
    MUXF8 f8_{dest_site2} (
        .I0(f7_to_f8_{dest_site2})
        );

    assign dest_wire3_{dest_site2} = {dest_wire3};
    assign dest_wire4_{dest_site2} = {dest_wire4};
    """.format(
            dest_site=dest_site,
            dest_site2=dest2_site,
            origin_site=origin_site,
            dest_wire=dest_wire,
            dest_wire2=dest_wire2,
            dest_wire3=dest_wire3,
            dest_wire4=dest_wire4,
        ))


def run():
    print('''
module top();
    ''')

    int_tiles = []

    db = Database(util.get_db_root(), util.get_part())
    grid = db.grid()
    params = {}
    for int_l_column, int_r_column in gen_orphan_ints(grid):
        for int_l, int_r in zip(int_l_column[1:6:2], int_r_column[1:6:2]):
            build_cross_int(params, grid, int_l, int_r)
            int_tiles.append(int_l)

    print("endmodule")
    with open('top.txt', 'w') as f:
        for tile in int_tiles:
            print(tile, file=f)


if __name__ == '__main__':
    run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
import argparse
import subprocess
import tempfile
import os.path
import glob


def main():
    parser = argparse.ArgumentParser(
        description=
        "Tool for comparing database segbits outputs from two database's.")
    parser.add_argument('a_db')
    parser.add_argument('b_db')

    args = parser.parse_args()

    assert os.path.isdir(args.a_db)
    assert os.path.isdir(args.b_db)

    for a_db_in in glob.glob('{}/segbits*.db'.format(args.a_db)):
        a_db_base = os.path.basename(a_db_in)
        b_db_in = '{}/{}'.format(args.b_db, a_db_base)

        if not os.path.exists(b_db_in):
            print('{} not found!'.format(b_db_in))
            continue

        with tempfile.NamedTemporaryFile(suffix="_a_{}".format(
                a_db_base)) as a_db_out, tempfile.NamedTemporaryFile(
                    suffix="_b_{}".format(a_db_base)) as b_db_out:
            subprocess.check_call(
                "sort {}".format(a_db_in), shell=True, stdout=a_db_out)
            subprocess.check_call(
                "sort {}".format(b_db_in), shell=True, stdout=b_db_out)

            print("Comparing {}".format(a_db_base))
            subprocess.call(
                "diff -U 0 {} {}".format(a_db_out.name, b_db_out.name),
                shell=True)


if __name__ == "__main__":
    main()
